import { kebabCase } from 'scule';
import chalk from 'chalk';
import Consola from 'consola';

const name = "@nuxtjs/design-tokens-edge";
const version = "0.0.1-27689964.f642c81";

const MODULE_DEFAULTS = {
  tokens: true,
  server: false
};
const _logger = Consola;
function useLogger(scope) {
  return scope ? _logger.withScope(scope) : logger;
}
const logger = useLogger("design-tokens");
chalk.magentaBright(name);
const objectPaths = (data) => {
  const output = [];
  function step(obj, prev) {
    Object.keys(obj).forEach((key) => {
      const value = obj[key];
      const isarray = Array.isArray(value);
      const type = Object.prototype.toString.call(value);
      const isobject = type === "[object Object]" || type === "[object Array]";
      const newKey = prev ? `${prev}.${key}` : key;
      if (!output.includes(newKey)) {
        output.push(newKey);
      }
      if (!isarray && isobject && Object.keys(value).length) {
        return step(value, newKey);
      }
    });
  }
  step(data);
  return output;
};
const get = (obj, path, defValue = void 0) => {
  if (!path) {
    return void 0;
  }
  const pathArray = Array.isArray(path) ? path : path.match(/([^[.\]])+/g);
  const result = pathArray.reduce(
    (prevObj, key) => prevObj && prevObj[key],
    obj
  );
  return result === void 0 ? defValue : result;
};

const referenceRegexDefaults = {
  opening_character: "{",
  closing_character: "}",
  separator: "."
};
function createReferenceRegex(opts = {}) {
  const options = Object.assign({}, referenceRegexDefaults, opts);
  return new RegExp(
    "\\" + options.opening_character + "([^" + options.closing_character + "]+)\\" + options.closing_character,
    "g"
  );
}
const resolveVariableFromPath = (path) => `var(--${path.split(".").map((key) => kebabCase(key)).join("-")})`;
const DesignTokenType = `interface DesignToken<T = string | number> {
  /* The raw value you specified in your token declaration. */
  value?: T;
  /* CSS Variable reference that gets generated. */
  variable?: string;
  name?: string;
  comment?: string;
  themeable?: boolean;
  property?: string;
  /* Is the property using compose() */
  composed?: boolean;
  /* Is the property using palette() */
  palette?: boolean
  /* Is the property nested in a selector */
  nested?: string;
  /* Is the property nested in variant */
  variant?: string;
  attributes?: {
    category?: string;
    type?: string;
    item?: string;
    subitem?: string;
    state?: string;
    [key: string]: any;
  };
  [key: string]: any;
}`;
const DesignTokensType = `interface DesignTokens {
  [key: string]: DesignTokens | DesignToken;
}`;
const TokenHelperOptionsType = `interface TokenHelperOptions {
  /**
   * The key that will be unwrapped from the design token object.
   * @default variable
   */
  key?: string
  /**
   * Toggle logging if requesting an unknown token.
   * @default false
   */
  silent?: boolean
  /**
   * Toggle deep flattening of the design token object to the requested key.
   *
   * If you query an token path containing mutliple design tokens and want a flat \`key: value\` object, this option will help you do that.
   */
  flatten?: boolean
}`;
const getFunction = `const get = ${get.toString()}`;
const treeWalker = (obj, typing = true, aliased = {}) => {
  let type = {};
  if (obj.value) {
    const _path = obj.path.join(".");
    const refRegex = createReferenceRegex({});
    const keyRegex = /{(.*)}/g;
    const testOriginal = obj.original.value.match(refRegex);
    if (testOriginal?.[0] && testOriginal[0] === obj.original.value) {
      obj.value = obj.original.value.replace(keyRegex, (_, tokenPath) => {
        aliased[_path] = resolveVariableFromPath(tokenPath);
        return aliased[_path];
      });
    }
    obj.variable = aliased?.[_path] ? aliased[_path] : `var(--${obj.name})`;
    type = typing ? "DesignToken" : obj;
  } else {
    for (const k in obj) {
      if (obj[k] && typeof obj[k] === "object") {
        type[k] = treeWalker(obj[k], typing, aliased).type;
      }
    }
  }
  return { type, aliased };
};
const tsTypesDeclaration = ({ tokens }) => {
  const { type } = treeWalker(tokens);
  let result = "import type { Ref\xA0} from 'vue'\n\n";
  result = result + `export ${DesignTokenType}

`;
  result = result + `export ${DesignTokensType}

`;
  result = result + `export ${TokenHelperOptionsType}

`;
  result = result + `export interface NuxtDesignTokens extends DesignTokens ${JSON.stringify(type, null, 2)}

`;
  const tokensPaths = objectPaths(type);
  if (tokensPaths.length) {
    result = result + `export type DesignTokensPaths = ${tokensPaths.map((path) => `'${path}'`).join(" | \n")}

`;
  } else {
    result = result + "export type DesignTokensPaths = 'no.tokens'\n\n";
  }
  result = result.replace(/"DesignToken"/g, "DesignToken");
  return result;
};
const tsFull = ({ tokens }) => {
  const { type, aliased } = treeWalker(tokens, false);
  let result = "import type { NuxtDesignTokens, DesignTokensPaths, DesignToken, TokenHelperOptions } from './types.d'\n\n";
  result = result + "export * from './types.d'\n\n";
  result = result + `${getFunction}

`;
  result = result + `export const tokenAliases = ${JSON.stringify(aliased, null, 2)} as const

`;
  result = result + `export const designTokens: NuxtDesignTokens = ${JSON.stringify(type, null, 2)}

`;
  result = result + `const defaultTokenHelperOptions: TokenHelperOptions = {
  key: 'variable',
  flatten: true,
  silent: false
}

/**
 * Get a theme token by its path
 */
export const $tokens = (path: DesignTokensPaths = undefined, options: TokenHelperOptions = {}) => {
  const { key, flatten } = Object.assign(defaultTokenHelperOptions, options)

  if (!path) return designTokens

  if (key === 'variable' && tokenAliases[path]) {
    return tokenAliases[path]
  }

  const token = get(designTokens, path)

  if (key && token?.[key]) { return token[key] }

  if (key && flatten && typeof token === 'object') {
    const flattened = {}
    
    const flattenTokens = (obj) => {
      Object.entries(obj).forEach(([objectKey, value]) => {
        if (value[key]) {
          flattened[objectKey] = value[key]
          return
        }

        flattenTokens(value)
      })
    }

    flattenTokens(token)

    return flattened
  }

  return token
}

`;
  result = result + "export const $dt = $tokens\n\n";
  return result;
};
const jsFull = ({ tokens }) => {
  const { type } = treeWalker(tokens, false);
  let result = `${getFunction}

`;
  result = result + `export const designTokens = ${JSON.stringify(type, null, 2)}

`;
  result = result + `const defaultTokenHelperOptions = {
  key: 'variable',
  flatten: true,
  silent: false
}

/**
 * Get a theme token by its path
 * @typedef {import('types').DesignTokensPaths} DesignTokensPaths
 * @typedef {import('types').DesignToken} DesignToken
 * * @typedef {import('types').TokenHelperOptions} TokenHelperOptions
 * @param {DesignTokensPaths} path The path to the theme token
 * @param {TokenHelperOptions} options Optional options
 */
export const $tokens = (path = undefined, options = {}) => {
  const { key, flatten } = Object.assign(defaultTokenHelperOptions, options)

  if (!path) return designTokens

  if (key === 'variable' && tokenAliases[path]) {
    return tokenAliases[path]
  }

  const token = get(designTokens, path)

  if (key && token?.[key]) { return token[key] }

  if (key && flatten && typeof token === 'object') {
    const flattened = {}
    
    const flatten = (obj) => {
      Object.entries(obj).forEach(([objectKey, value]) => {
        if (value[key]) {
          flattened[objectKey] = value[key]
          return
        }

        flatten(value)
      })
    }

    flatten(token)

    return flattened
  }

  return token
}

`;
  result = result + "export const $dt = $tokens\n\n";
  return result;
};
const nuxtPlugin = ({ tokens }) => {
  const { aliased } = treeWalker(tokens, false);
  return `import type { Ref } from 'vue'
  import { kebabCase } from 'scule'
  import type { DesignTokensPaths } from '#design-tokens/types'
  import { defineNuxtPlugin, unref } from '#imports'
  
  export const tokenAliases = ${JSON.stringify(aliased, null, 2)}

  export default defineNuxtPlugin(() => {
    const resolveVariableFromPath = (path: DesignTokensPaths | Ref<DesignTokensPaths>): string => {
      if (tokenAliases[path]) { return tokenAliases[path] }
      return \`var(--\${unref(path).split('.').map(key => kebabCase(key)).join('-')})\`
    }
  
    return {
      provide: {
        tokens: resolveVariableFromPath,
        dt: resolveVariableFromPath
      }
    }
  })`;
};

export { MODULE_DEFAULTS as M, treeWalker as a, tsFull as b, nuxtPlugin as c, jsFull as j, logger as l, name as n, tsTypesDeclaration as t, version as v };
