import { existsSync } from 'fs';
import { mkdir, rm, writeFile } from 'fs/promises';
import StyleDictionary from 'browser-style-dictionary/browser.js';
import { M as MODULE_DEFAULTS, t as tsTypesDeclaration, a as treeWalker, b as tsFull, j as jsFull } from './formats.mjs';
import { requireModule } from '@nuxt/kit';
import { resolve } from 'pathe';
import { defu } from 'defu';

const resolveConfig = (layer, key, configFile = `${key}.config`) => {
  const value = layer.config?.designTokens?.[key] || MODULE_DEFAULTS[key];
  let config = {};
  let filePath;
  if (typeof value === "boolean") {
    filePath = resolve(layer.cwd, configFile);
  } else if (typeof value === "string") {
    filePath = resolve(layer.cwd, value);
  } else if (typeof value === "object") {
    config = value;
  }
  if (filePath) {
    try {
      const _file = requireModule(filePath, { clearCache: true });
      if (_file) {
        config = _file;
      }
    } catch (_) {
    }
  }
  return { filePath, config };
};
const resolveConfigTokens = (layers) => {
  const tokensFilePaths = [];
  let tokens = {};
  const splitLayer = (layer) => {
    if (layer.config?.designTokens?.tokens || MODULE_DEFAULTS.tokens) {
      const { config: layerTokens, filePath: _layerTokensFilePath } = resolveConfig(layer, "tokens", "tokens.config");
      if (_layerTokensFilePath) {
        tokensFilePaths.push(_layerTokensFilePath);
      }
      tokens = defu(tokens, layerTokens);
    }
  };
  for (const layer of layers) {
    splitLayer(layer);
  }
  return { tokensFilePaths, tokens };
};
const createTokensDir = async (path) => {
  if (!existsSync(path)) {
    await mkdir(path, { recursive: true });
  }
};

const stubTokens = async (buildPath, force = false) => {
  const files = {
    "__uses-tokens.ts": () => "",
    "tokens.css": () => "/* This file is empty because no tokens has been provided. */",
    "tokens.scss": () => "/* This file is empty because no tokens has been provided. */",
    "tokens.json": () => "{}",
    "index.js": jsFull,
    "index.ts": tsFull,
    "types.d.ts": tsTypesDeclaration
  };
  for (const [file, stubbingFunction] of Object.entries(files)) {
    const path = `${buildPath}${file}`;
    if (force && existsSync(path)) {
      await rm(path);
    }
    if (!existsSync(path)) {
      await writeFile(path, stubbingFunction ? stubbingFunction({ tokens: {}, allTokens: [] }) : "");
    }
  }
};
const getStyleDictionaryInstance = async (tokens, buildPath) => {
  let styleDictionary = StyleDictionary;
  styleDictionary.fileHeader = {};
  styleDictionary.registerTransformGroup({
    name: "tokens-js",
    transforms: ["name/cti/kebab", "size/px", "color/hex"]
  });
  styleDictionary.registerFormat({
    name: "typescript/types-declaration",
    formatter({ dictionary }) {
      return tsTypesDeclaration(dictionary);
    }
  });
  styleDictionary.registerFormat({
    name: "css/full",
    formatter({ dictionary, options }) {
      const selector = options.selector ? options.selector : ":root";
      const { outputReferences } = options;
      const { formattedVariables } = StyleDictionary.formatHelpers;
      const { aliased } = treeWalker(dictionary.tokens, false);
      dictionary.allTokens = dictionary.allTokens.filter((token) => !aliased[token.path.join(".")]);
      return `${selector} {
` + formattedVariables({ format: "css", dictionary, outputReferences }) + "\n}\n";
    }
  });
  styleDictionary.registerFormat({
    name: "typescript/full",
    formatter({ dictionary }) {
      return tsFull(dictionary);
    }
  });
  styleDictionary.registerFormat({
    name: "javascript/full",
    formatter({ dictionary }) {
      return jsFull(dictionary);
    }
  });
  styleDictionary = await styleDictionary.extend({
    tokens,
    platforms: {
      scss: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.scss",
            format: "scss/variables"
          }
        ]
      },
      json: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.json",
            format: "json/flat"
          }
        ]
      },
      ts: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "index.ts",
            format: "typescript/full"
          },
          {
            destination: "types.d.ts",
            format: "typescript/types-declaration"
          }
        ]
      },
      js: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "index.js",
            format: "javascript/full"
          }
        ]
      },
      css: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.css",
            format: "css/full"
          }
        ]
      },
      done: {
        actions: ["done"]
      }
    }
  });
  return styleDictionary;
};
const generateTokensOutputs = (styleDictionary, silent = true) => new Promise(
  (resolve, reject) => {
    try {
      if (silent) {
        console._log = console.log;
        console.log = () => {
        };
      }
      styleDictionary.registerAction({
        name: "done",
        do: (dictionary) => {
          resolve(dictionary);
        },
        undo: () => {
        }
      });
      styleDictionary.cleanAllPlatforms();
      styleDictionary.buildAllPlatforms();
      if (silent) {
        console.log = console._log;
      }
    } catch (e) {
      reject(e);
    }
  }
);
const generateTokens = async (tokens, buildPath, silent = false) => {
  if (!existsSync(buildPath)) {
    await createTokensDir(buildPath);
  }
  if (!tokens || !Object.keys(tokens).length) {
    await stubTokens(buildPath, true);
    return;
  }
  const styleDictionary = await getStyleDictionaryInstance(tokens, buildPath);
  return await generateTokensOutputs(styleDictionary, silent);
};

export { getStyleDictionaryInstance as a, createTokensDir as c, generateTokens as g, resolveConfigTokens as r, stubTokens as s };
