import { performance } from 'perf_hooks';
import { readFile } from 'fs/promises';
import { defineNuxtModule, createResolver, addPluginTemplate, resolveModule, addPlugin, addAutoImport } from '@nuxt/kit';
import { withTrailingSlash } from 'ufo';
import { defu } from 'defu';
import { join } from 'pathe';
import { c as createTokensDir, g as generateTokens, r as resolveConfigTokens } from './chunks/generate.mjs';
import { l as logger, n as name, v as version, M as MODULE_DEFAULTS } from './chunks/formats.mjs';
import { createUnplugin } from 'unplugin';
import { kebabCase } from 'scule';
import chalk from 'chalk';
import { $tokens } from './index.mjs';
import 'fs';
import 'browser-style-dictionary/browser.js';
import 'consola';
import 'jiti';
import 'chroma-js';

const variantPropsRegex = /\$variantProps\('(.*)'\)/g;
const screenRegex = /(@screen\s(.*?)\s{)/g;
const darkRegex = /(@dark\s{)/g;
const lightRegex = /(@light\s{)/g;
const dtRegex = /\$dt\('(.*?)'\)/g;
const componentRegex = /@component\('(.*?)'\);/g;
const resolveDt = (code, wrapper = void 0) => {
  const replace = (path) => `${wrapper || ""}var(--${path.split(".").map((key) => kebabCase(key)).join("-")})${wrapper || ""}`;
  code = code.replace(dtRegex, (_, path) => replace(path));
  return code;
};
const resolveVariantProps = (code) => {
  code = code.replace(variantPropsRegex, (_, componentName) => {
    const variants = $tokens("components." + componentName + ".variants", { key: void 0, flatten: false, silent: true });
    if (!variants) {
      return `{
        variants: {
          type: Array,
          required: false,
          default: []
        }
      }`;
    }
    return `{
      variants: {
        type: Array as import('vue').PropType<${Object.keys(variants).map((variant) => `'${variant}'`).join(" | ")}>,
        required: false,
        default: []
      }
    }`;
  });
  return code;
};
const resolveScheme = (code, scheme) => {
  const schemesRegex = {
    lightRegex,
    darkRegex
  };
  code = code.replace(schemesRegex[scheme + "Regex"], `@media (prefers-color-scheme: ${scheme}) {`);
  return code;
};
const resolveScreen = (code, id) => {
  const screens = $tokens("screens", {
    flatten: false,
    key: void 0,
    silent: true
  });
  if (!screens) {
    console.log("`@screens` directive needs you to specify a `screens` set in your tokens.config file.");
    return code;
  }
  code = code.replace(screenRegex, (_, _screenDeclaration, screenSize) => {
    const screenToken = screens[screenSize];
    if (screenToken) {
      return `@media (min-width: ${screenToken.value}) {`;
    }
    logger.warn(
      "Error in component:",
      id,
      "This screen size is not defined: " + chalk.red(screenSize) + "\n\n",
      "Available screen sizes: ",
      Object.keys(screens).map((screen) => chalk.green(screen)).join(", ")
    );
    return "@media (min-width: 0px) {";
  });
  return code;
};
const resolveComponent = (code) => {
  code = code.replace(componentRegex, (_, componentName) => {
    const component = $tokens("components." + componentName, { key: void 0, flatten: false, silent: true });
    if (component && Object.keys(component)) {
      const styleDeclaration = resolveStyleFromComponent(component);
      return styleDeclaration;
    }
    return "";
  });
  return code;
};
const resolveStyleFromComponent = (component) => {
  let styleDeclaration = "";
  const styles = {
    root: {
      propertiesLines: [],
      nestings: []
    }
  };
  const resolvePropertiesObject = (value, target = styles.root) => {
    Object.entries(value).forEach(([_key, _value]) => {
      const { variable, pseudo, property } = _value;
      if (_key === "variants") {
        Object.entries(_value).forEach(([_variantKey, _variantValue]) => {
          if (!styles[_variantKey]) {
            styles[_variantKey] = {
              nestings: [],
              propertiesLines: []
            };
          }
          resolvePropertiesObject(_variantValue, styles[_variantKey]);
        });
        return;
      }
      if (pseudo) {
        if (!target.nestings[pseudo]) {
          target.nestings[pseudo] = [];
        }
        target.nestings[pseudo].push(`${property}: ${variable};`);
        return;
      }
      target.propertiesLines.push(`${property}: ${variable};`);
    }, {});
  };
  resolvePropertiesObject(component);
  Object.entries(styles).forEach(
    ([key, { propertiesLines, nestings }]) => {
      let localDeclaration = "";
      localDeclaration = propertiesLines.join("\n");
      Object.entries(nestings).forEach(([nested, _propertiesLines]) => {
        localDeclaration += `
&${nested} {
${_propertiesLines.join("\n")}
}`;
      });
      key === "root" ? styleDeclaration += localDeclaration : styleDeclaration += `&.${key} {
${localDeclaration}
}`;
    }
  );
  return styleDeclaration;
};

const jsFilesRegex = /\.((c|m)?j|t)sx?$/g;
const vueRootRegex = /.vue(?!&type=style)/;
const vueStyleRegex = /vue&type=style/;
const templateRegex = /<template.*>(.|\n)*?<\/template>/g;
const scriptRegex = /<script.*>(.|\n)*?<\/script>/g;
const unpluginDesignTokensStyle = createUnplugin(({ hasTailwind }) => {
  return {
    name: "unplugin-nuxt-design-tokens-style",
    enforce: "pre",
    transformInclude(id) {
      return vueStyleRegex.test(id);
    },
    transform(code, id) {
      code = resolveDt(code);
      if (hasTailwind) {
        code = resolveScreen(code, id);
      }
      code = resolveScheme(code, "dark");
      code = resolveScheme(code, "light");
      code = resolveComponent(code);
      return { code };
    }
  };
});
const unpluginDesignTokensComponents = createUnplugin(() => {
  return {
    name: "unplugin-nuxt-design-tokens-components",
    enforce: "pre",
    transformInclude(id) {
      return vueRootRegex.test(id) || jsFilesRegex.test(id);
    },
    transform(code) {
      code = code.replace(
        templateRegex,
        (code2) => resolveDt(code2, "'")
      );
      code = code.replace(
        scriptRegex,
        (code2) => {
          code2 = resolveDt(code2, "'");
          return resolveVariantProps(code2);
        }
      );
      return { code };
    }
  };
});
const registerTransformPlugin = (nuxt, options) => {
  nuxt.hook("webpack:config", (config) => {
    config.plugins = config.plugins || [];
    config.plugins.unshift(
      unpluginDesignTokensStyle.webpack(options),
      unpluginDesignTokensComponents.webpack(options)
    );
  });
  nuxt.hook("vite:extend", (vite) => {
    vite.config.plugins = vite.config.plugins || [];
    vite.config.plugins.push(
      unpluginDesignTokensStyle.vite(options),
      unpluginDesignTokensComponents.vite(options)
    );
  });
};

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "designTokens",
    compatibility: {
      nuxt: "^3.0.0-rc.5"
    }
  },
  defaults: MODULE_DEFAULTS,
  async setup(moduleOptions, nuxt) {
    nuxt.options.runtimeConfig.designTokens = {
      tokensFilePaths: [],
      server: moduleOptions.server
    };
    let hasTailwind = false;
    nuxt.hook("tailwindcss:config", () => {
      hasTailwind = true;
    });
    let lastTokensCount = 0;
    const layers = nuxt.options._layers;
    const privateConfig = nuxt.options.runtimeConfig.designTokens;
    const tokensDir = withTrailingSlash(nuxt.options.buildDir + "/tokens");
    globalThis.__nuxtDesignTokensBuildDir__ = tokensDir;
    const { resolve: resolveTokensDir } = createResolver(tokensDir);
    privateConfig.tokensDir = tokensDir;
    const { resolve: resolveRuntime } = createResolver(import.meta.url);
    const resolveRuntimeModule = (path) => resolveModule(path, { paths: resolveRuntime("./runtime") });
    await createTokensDir(tokensDir);
    const postcssOptions = nuxt.options.postcss || nuxt.options.build.postcss.postcssOptions || nuxt.options.build.postcss;
    postcssOptions.plugins = postcssOptions.plugins || {};
    postcssOptions.plugins["postcss-nested"] = postcssOptions.plugins["postcss-nested"] ?? {};
    postcssOptions.plugins["postcss-custom-properties"] = postcssOptions.plugins["postcss-custom-properties"] ?? {};
    const refreshTokens = async (nitro) => {
      const { tokensFilePaths: tokensFilePaths2, tokens: tokens2 } = resolveConfigTokens(layers);
      if (moduleOptions.tokens) {
        privateConfig.tokensFilePaths = tokensFilePaths2;
      }
      if (nitro) {
        const projectConfigPath = resolveModule(tokensFilePaths2[0]);
        const projectConfig = (await readFile(projectConfigPath)).toString();
        await nitro.storage.setItem("cache:design-tokens:config", projectConfig);
        await nitro.storage.setItem("cache:design-tokens:tokens.json", tokens2);
      }
      return { tokens: tokens2, tokensFilePaths: tokensFilePaths2 };
    };
    const buildTokens = async (nitro, viteServer) => {
      try {
        const start = performance.now();
        const tokens2 = await nitro.storage.getItem("cache:design-tokens:tokens.json");
        const dictionary = await generateTokens(tokens2, tokensDir);
        const end = performance.now();
        const ms = Math.round(end - start);
        logger.success(`Design Tokens built${ms ? " in " + ms + "ms" : ""}`);
        if (viteServer && lastTokensCount !== dictionary.allTokens.length) {
          lastTokensCount = dictionary.allTokens.length;
          viteServer.moduleGraph.invalidateAll();
          viteServer.ws.send({
            type: "full-reload",
            path: "*"
          });
        }
      } catch (e) {
        logger.error("Could not build tokens!");
        logger.error(e.message);
      }
    };
    const { tokens, tokensFilePaths } = await refreshTokens();
    nuxt.options.build.transpile = nuxt.options.build.transpile || [];
    nuxt.options.build.transpile.push(resolveRuntime("./runtime"));
    nuxt.options.alias = nuxt.options.alias || {};
    nuxt.options.alias["#design-tokens"] = resolveTokensDir("index");
    nuxt.options.alias["#design-tokens/style"] = resolveTokensDir("tokens.css");
    nuxt.options.alias["#design-tokens/types"] = resolveTokensDir("types.d");
    addPluginTemplate({
      filename: "tokens.mjs",
      getContents: () => {
        const lines = [
          `import '${resolveTokensDir("tokens.css")}'`,
          `import '${resolveRuntime("./runtime/assets/reset.css")}'`,
          "export default defineNuxtPlugin(() => {})"
        ];
        return lines.join("\n");
      }
    });
    registerTransformPlugin(nuxt, { tokensDir, tokensFilePaths: tokensFilePaths.map((path) => resolveModule(path)), hasTailwind });
    nuxt.hook("prepare:types", (opts) => {
      opts.references.push({ path: resolveTokensDir("types.d.ts") });
    });
    nuxt.hook("nitro:build:before", async (nitro) => {
      await refreshTokens(nitro);
      await buildTokens(nitro);
    });
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.handlers = nitroConfig.handlers || [];
      nitroConfig.handlers.push({
        method: "all",
        route: "/api/_design-tokens/tokens",
        handler: resolveRuntimeModule("./server/api/tokens/index")
      });
      nitroConfig.handlers.push({
        method: "all",
        route: "/api/_design-tokens/config",
        handler: resolveRuntimeModule("./server/api/tokens/config")
      });
      nitroConfig.prerender = nitroConfig.prerender || {};
      nitroConfig.prerender.routes = nitroConfig.prerender.routes || [];
      nitroConfig.prerender.routes.push("/api/_design-tokens/tokens");
      nitroConfig.prerender.routes.push("/api/_design-tokens/config");
      nitroConfig.bundledStorage = nitroConfig.bundledStorage || [];
      nitroConfig.bundledStorage.push("/cache/tokens");
      nitroConfig.bundledStorage.push("/tokens");
      if (privateConfig.server) {
        nitroConfig.handlers.push({
          method: "get",
          route: "/api/_design-tokens/tokens/generate",
          handler: resolveRuntimeModule("./server/api/tokens/generate")
        });
        nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
          inline: [
            "lodash",
            "browser-style-dictionary",
            "browser-style-dictionary/lib",
            "browser-style-dictionary/browser",
            resolveRuntime()
          ]
        });
        nitroConfig.alias["#design-tokens"] = resolveRuntime("./runtime/server/utils");
      }
    });
    addPlugin({
      src: resolveRuntimeModule("./plugins/tokens")
    });
    addAutoImport({
      from: resolveRuntimeModule("./composables/tokens"),
      name: "useTokens",
      as: "useTokens"
    });
    if (nuxt.options.dev) {
      nuxt.hook("nitro:init", (nitro) => {
        nuxt.hook("vite:serverCreated", (viteServer) => {
          tokensFilePaths.forEach((path) => viteServer.config.assetsInclude(resolveModule(path)));
          nuxt.hook("builder:watch", async (_, path) => {
            const extensions = [".js", ".ts", ".mjs"];
            const isTokensConfig = extensions.some((extension) => path.includes("tokens.config" + extension));
            if (isTokensConfig) {
              const { tokens: tokens2 } = await refreshTokens(nitro);
              viteServer.ws.send({
                type: "custom",
                event: "nuxt-design-tokens:update",
                data: {
                  tokens: moduleOptions.tokens ? tokens2 : {}
                }
              });
              await buildTokens(nitro, viteServer);
            }
          });
        });
      });
    }
    nuxt.hook("build:before", async () => await generateTokens(tokens, tokensDir, true));
    await generateTokens(tokens, tokensDir, true);
    nuxt.hook("tailwindcss:config", (tailwindConfig) => {
      tailwindConfig.content = tailwindConfig.content ?? [];
      tailwindConfig.content.push(join(tokensDir, "index.ts"));
    });
  }
});

export { module as default };
